@@ -18,7 +18,7 @@
 
      For further information visit http://plib.sourceforge.net
 
-     $Id: viewer.cxx,v 1.23 2002/09/01 12:04:52 ude Exp $
+     $Id: viewer.cxx 2099 2006-11-03 20:04:40Z fayjf $
 */
 
 /****
@@ -59,6 +59,7 @@
 #include <plib/ssg.h>
 #include <plib/fnt.h>
 #include <plib/pu.h>
+#include <plib/puAux.h>
 
 #define ARROWS_USED 1
 /*
@@ -69,10 +70,13 @@ Defines what type of arrows to uses with puFileSelector
 2 - Single move and Jump 10 moves
 */
 
+
+
 /*
 scene graph
 */
 static ssgRoot *scene = NULL ;
+static ssgContext *context = NULL ;
 static ssgEntity* camera_object = NULL ;
 
 /*
@@ -81,7 +85,7 @@ font vars
 static fntRenderer *text ;
 static fntTexFont *font ;
 
-static puFileSelector* file_selector = 0 ;
+static puaFileSelector* file_selector = 0 ;
 
 /*
 frame rate vars
@@ -96,9 +100,8 @@ animation vars
 static const int speed_tab [] =
 { 0, 1, 2, 3, 4, 5, 6, 10, 12, 15, 20, 30, 60 };
 
-static int speed_index = 0 ;
-static int anim_delay = -1 ;
-static int anim_frame ;
+static int speed_index = 1 ;
+//static int anim_frame ;
 static int num_anim_frames ;
 
 /*
@@ -142,8 +145,13 @@ static GLfloat Ex     = 0.0f;
 static GLfloat Ey     = 0.0f;
 static GLfloat Ez     = 0.0f;
 
-//#define FOV 60.0f
-#define FOV 45.0f
+#define FOVY 45.0f
+#define FOV_NEAR 0.1f
+#define FOV_FAR  10000.0f
+
+#ifndef M_PI
+#define M_PI   3.14159265358979323846264338327950f
+#endif
 
 static int getWindowHeight () { return glutGet ( (GLenum) GLUT_WINDOW_HEIGHT ) ; }
 static int getWindowWidth  () { return glutGet ( (GLenum) GLUT_WINDOW_WIDTH  ) ; }
@@ -231,14 +239,25 @@ static void count_anim_frames( ssgEntity *e )
 }
 
 
-static void set_anim_frame( ssgEntity *e )
+
+// You need to have a object with animation loaded into memory for the following function to make sense.
+// Animation can be discreet (in steps) or "smooth" (fluid).
+// If you loaded an animation made of several meshes, it will use a selector and therefore always be "discreet"
+// If you loaded some transformation matrices like some rotations to rotate the flap of a plane,
+// the define DISCREET_STEPS will determine whether the animation is smooth (new code, WK) or 
+// in steps ("old" code, Dave McClurg). 
+//
+// The anim_frame is for discreet animation and an index determining which matrix / which selector will be used.
+//
+// typically, you will call set_anim_frame directly before you call ssgCullAndDraw 
+static void set_anim_frame( ssgEntity *e, int anim_frame )
 {
   if ( e -> isAKindOf ( ssgTypeBranch() ) )
   {
     ssgBranch *br = (ssgBranch *) e ;
     
     for ( int i = 0 ; i < br -> getNumKids () ; i++ )
-      set_anim_frame ( br -> getKid ( i ) ) ;
+      set_anim_frame ( br -> getKid ( i ), anim_frame ) ;
     
     if ( e -> isAKindOf ( ssgTypeSelector() ) )
     {
@@ -247,29 +266,10 @@ static void set_anim_frame( ssgEntity *e )
       if ( num > 0 )
       {
         int frame = anim_frame ;
-        if ( frame >= num )
-          frame = num-1 ;
+        frame %= num ;
         p -> selectStep ( frame ) ;
       }
     }
-    else if ( e -> isAKindOf ( ssgTypeTransform() ) )
-    {
-      ssgBase* data = e -> getUserData () ;
-      if ( data != NULL && data -> isAKindOf ( ssgTypeTransformArray() ) )
-      {
-        ssgTransform* p = (ssgTransform*) e ;
-        ssgTransformArray* ta = (ssgTransformArray*) data ;
-        int num = ta -> getNum () ;
-        if ( num > 0 )
-        {
-          int frame = anim_frame ;
-          if ( frame >= num )
-            frame = num-1 ;
-          ta -> selection = frame ;
-          p -> setTransform ( *( ta -> get ( ta -> selection ) ) ) ;
-        }
-      }
-    }
   }
   else if ( e -> isAKindOf ( ssgTypeLeaf() ) )
   {
@@ -283,8 +283,7 @@ static void set_anim_frame( ssgEntity *e )
       if ( num > 0 )
       {
         int frame = anim_frame ;
-        if ( frame >= num )
-          frame = num-1 ;
+        frame %= num ;
         ss -> selectStep ( frame ) ;
       }
     }
@@ -292,6 +291,7 @@ static void set_anim_frame( ssgEntity *e )
 }
 
 
+
 static int wire_draw ( ssgEntity* e )
 {
   if ( e -> isAKindOf ( ssgTypeLeaf() ) )
@@ -388,14 +388,23 @@ static void follow_camera ( sgMat4 mat )
 
 /*
   The GLUT window reshape event
+  Fixed to properly maintain aspect ratio.
 */
 
 static void reshape ( int w, int h )
 {
   glViewport ( 0, 0, w, h ) ;
+
+  float aspect = w / (float)h;
+  float angle = 0.5f * FOVY * M_PI / 180.0f;
+  float y = FOV_NEAR * (float)tan(angle);
+  float x = aspect * y;
+
+  context->setFrustum(-x,x,-y,y,FOV_NEAR,FOV_FAR);
 }
 
 
+
 /*
   The GLUT display event
 */
@@ -408,31 +417,26 @@ static void display(void)
   sgMat4 mat ;
   make_matrix ( mat ) ;
   follow_camera ( mat ) ;
-  ssgSetCamera ( mat );
+
+  context->setCamera ( mat );
   
   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
   glEnable(GL_BLEND);
   glEnable( GL_DEPTH_TEST ) ;
   
   /*
-  increment the frame counter
+  update virtual time, from "real" time and speed.
   */
   static int last_time = 0;
   int curr_time = glutGet((GLenum)GLUT_ELAPSED_TIME);
-  while (curr_time >= last_time + 1000/60)
-  {
-    last_time += 1000/60;
-
-    static int anim_time = 0 ;
-    if ( anim_delay != -1 && ++ anim_time >= anim_delay )
-    {
-      if ( ++ anim_frame >= num_anim_frames )
-        anim_frame = 0 ;
-      set_anim_frame ( scene ) ;
-      anim_time = 0 ;
-    }
-  }
+  if (last_time != 0)
+    // calculate the "virtual time"  
+    _ssgGlobTime += 0.001f*(curr_time-last_time)*speed_tab[ speed_index ];
 
+  last_time = curr_time;
+  
+  //set_anim_frame ( scene, _ssgGlobTime); // currently not needed by our sample data 
+ 
   /*
   figure out how fast this bus is going
   */
@@ -474,6 +478,7 @@ static void display(void)
   }
   else if ( fps > 0 )
   {
+    
     char buffer [ PUSTRING_MAX ] ;
     text -> begin () ;
     glColor3f ( 0.0f, 0.0f, 0.0f ) ;
@@ -484,9 +489,10 @@ static void display(void)
       text -> puts ( buffer ) ;
     sprintf ( buffer, "anim_speed : %d fps\n", speed_tab[ speed_index ] ) ;
       text -> puts ( buffer ) ;
-    sprintf ( buffer, "time : %-4d, %d\n", curr_time, anim_frame ) ;
+    sprintf ( buffer, "time : %-4d\n", curr_time ) ;
       text -> puts ( buffer ) ;
     text -> end () ;
+    
   }
   
   glColor3f ( 1.0f, 1.0f, 1.0f ) ;
@@ -670,7 +676,9 @@ static void pick_cb ( puObject * )
   wire_update ( scene, wire_flag ) ;
   
   SGfloat radius = scene->getBSphere()->getRadius();
-  EyeDist = float( radius * 1.5f / tan( float( FOV/2 * SG_DEGREES_TO_RADIANS ) ) );
+  EyeDist = float( radius * 1.5f / tan( float( FOVY/2 * SG_DEGREES_TO_RADIANS ) ) );
+  if (EyeDist < FOV_NEAR*2)
+    EyeDist = FOV_NEAR*2;
   
   sgSphere sp = *( scene -> getBSphere() ) ;
   if ( sp.isEmpty() )
@@ -691,14 +699,6 @@ static void pick_cb ( puObject * )
 }
 
 
-static int get_delay( int speed_index )
-{
-  int fps = speed_tab[ speed_index ] ;
-  if ( fps > 0 )
-    return MAX_SPEED / fps ;
-  return -1 ;
-}
-
 
 /*
   The GLUT keyboard event
@@ -716,12 +716,10 @@ static void keyboard(unsigned char key, int, int)
   case '+':
     if ( speed_tab[ speed_index ] != MAX_SPEED )
       speed_index ++ ;
-    anim_delay = get_delay( speed_index );
     break;
   case '-':
     if ( speed_tab[ speed_index ] != 0 )
       speed_index -- ;
-    anim_delay = get_delay( speed_index );
     break;
   case ' ':
     Ex = 0;
@@ -731,7 +729,7 @@ static void keyboard(unsigned char key, int, int)
   case 'l':
     if ( !file_selector )
     {
-      file_selector = new puFileSelector ( ( 640 - 320 ) / 2, ( 480 - 270 ) / 2, 320, 270, ARROWS_USED, "data" ) ;
+      file_selector = new puaFileSelector ( ( 640 - 320 ) / 2, ( 480 - 270 ) / 2, 320, 270, ARROWS_USED, "data" ) ;
       file_selector -> setCallback ( pick_cb ) ;
     }
     break;
@@ -812,13 +810,6 @@ static void init_graphics ()
   ssgInit () ;
   puInit () ;
   
-  /*
-  Set up the viewing parameters
-  */
-  
-  ssgSetFOV     ( FOV, 0.0f ) ;
-  ssgSetNearFar ( 2.0f, 10000.0f ) ;
-  
   /*
     Set up the Sun.
   */
@@ -826,6 +817,8 @@ static void init_graphics ()
   sgVec3 sunposn ;
   sgSetVec3 ( sunposn, 200.0f, -500.0f, 500.0f ) ;
   ssgGetLight ( 0 ) -> setPosition ( sunposn ) ;
+
+
   
   /*
   Set up the path to the data files
@@ -839,6 +832,9 @@ static void init_graphics ()
   */
   
   scene = new ssgRoot      ;
+  context = new ssgContext ;
+  context->setCullface( true ) ;
+  context->makeCurrent ( ) ;
   
   /*
   ** Set up PU
